from ..widgets.dynamic_widgets import DynamicSwitch

try:

    from IPython.display import display
    import ipywidgets as widgets
    from ipywidgets import HBox, Label, VBox
    from ipywidgets import Button, Layout, HTML

except ModuleNotFoundError:

    pass

def get_control_panel(access_object):

    def on_update_topic_choice(*args):

        tab_children = []

        for idx, topic_name in enumerate(topic_choice.value):

            vbox = access_object.param_vboxes[topic_name]
            
            tab.set_title(idx, topic_name)
            tab_children.append(vbox)

        tab.children = tab_children
    
    style = style = {'description_width': 'initial'}

    topics = access_object.get_topics()

    topic_choice = widgets.SelectMultiple(
        options=topics,
        value=topics,
        rows=len(topics) + 1,
        disabled=False,
        style=style
    )

    label = widgets.Label(value='Select modes (keep shift and/or ctrl (or command) pressed)')

    topic_choice_box = VBox()
    topic_choice_box.children = [label, topic_choice]

    
    tab = widgets.Tab()

    tab_box = VBox()
    tab_box.children = [tab]
    
    topic_choice.observe(on_update_topic_choice)
    
    tab_children = []

    for idx, topic_name in enumerate(topic_choice.value):

        vbox = access_object.param_vboxes[topic_name]

        tab.set_title(idx, topic_name)
        tab_children.append(vbox)

    tab.children = tab_children
    
    v = DynamicSwitch(topic_choice_box, tab_box)
    
    return v




from ..tree.tree_node import TreeNode
from ..widgets.dynamic_widgets import DynamicWidget
from ..widgets.dynamic_widgets import DynamicSwitch

try:

    from IPython.display import display
    import ipywidgets as widgets
    from ipywidgets import HBox, Label, VBox
    from ipywidgets import Button, Layout, HTML

    module_found = True

except ModuleNotFoundError:

    module_found = False

from collections import defaultdict


class AccessObject():
    
    def __init__(self, root, node_dicts):


        self.module_found = module_found

        if not self.module_found:
            return
        
        self.root, self.node_dicts = root, node_dicts
        self.widget_dicts = defaultdict(dict)
        self.param_vboxes = {}

        for topic in self.root.children:

            param_widgets = []

            for param in topic.children:

                param_widget = DynamicWidget(topic.name, param, self.widget_dicts)

                param_widgets.append(param_widget)

            param_vbox = VBox(param_widgets)

            self.param_vboxes[topic.name] = param_vbox

    def get_topics(self):
    
        return self.root.get_children_names()

    def get_params(self, topic=None):

        if topic:
    
            return list(self.widget_dicts[topic].keys())

        else:

            l = []
            self._find_params(self.root, l)

            return l

    def _find_params(self, node, l):

        depth = node.depth
        node_type = node.primary_type
        node_name = node.name

        if node_type != 'root':

            if node_type == 'topic':

                depth += 1
                
            if node_type in ['param', 'optional']:
                
                if node_name not in l:
                    l.append(node_name)

        for child in node.children:

            self._find_params(child, l)

    def get_value(self, param, topic=None):
        
        return self.get_widget(param, topic).value

            
    def get_vbox(self, topic):
        
        return self.param_vboxes[topic]
    
    def get_widget(self, param, topic=None):
        
        if topic:
            
            return self.widget_dicts[topic][param]
        
        else:
            
            params = []
            topics = []
            
            for topic, param_dict in self.widget_dicts.items():
            
                if param in param_dict:
                    
                    params.append(param_dict[param])
                    topics.append(topic)
                    
            if len(params) > 1:
                
                raise TypeError('Specify the topic!', topics)
                
            return params[0]
        
    def get_node(self, node, topic=None):
        
        if topic:
            
            return self.node_dicts[topic][node]
        
        else:
            
            nodes = []
            topics = []
            
            for topic, node_dict in self.node_dicts.items():
                
                if node in node_dict:
                    
                    nodes.append(node_dict[node])
                    topics.append(topic)
                    
            if len(nodes) > 1:
                
                raise TypeError('Specify the topic!', topics)
                    
                    
            return nodes[0]




class TreeNode():
    
    def __init__(self, depth, name, default_value, primary_type, secondary_type=None):
        
        self.depth = depth
        self.name = name
        self.default_value = default_value
        
        self.primary_type = primary_type
        self.secondary_type = secondary_type
        
        self.children = []
        
    def add_child(self, node):
        
        self.children.append(node)
        
    def get_children_names(self):
        
        names = []
        
        for node in self.children:
            
            names.append(node.name)
            
        return names
    
    def get_child_by_name(self, name):
        
        for node in self.children:
            
            if name == node.name:
                return node
            
        return None
        
    def __repr__(self):
        return ('{}_{}_{}'.format(self.name, self.primary_type, self.secondary_type))




from .widgets import ParamChoiceWidget
from .widgets import Custom1
from .widgets import ParamTextWidget
from .widgets import ParamSetterWidget

try:

    from IPython.display import display
    import ipywidgets as widgets
    from ipywidgets import HBox, Label, VBox
    from ipywidgets import Button, Layout, HTML
    from traitlets import MetaHasTraits

except ModuleNotFoundError:

    class VBox():
        pass

    class MetaHasTraits():
        pass


class DynamicWidget(VBox):
    # https://stackoverflow.com/questions/60998665/is-it-possible-to-make-another-ipywidgets-widget-appear-based-on-dropdown-select
    
    def __init__(self, topic, node, widget_dicts):

        if not isinstance(VBox, MetaHasTraits):
            return
        
        self.topic = topic
        self.node = node
        
        self.widget_dicts = widget_dicts

        if node.primary_type=='param' or node.primary_type=='optional':

            is_optional_param = node.primary_type=='optional'

            # if choiceable param, add choices here
            if node.secondary_type=='option':
                
                options = node.get_children_names() 

                default_value = node.default_value

                if self.node.name in self.widget_dicts[self.topic]:

                    self.widget = self.widget_dicts[self.topic][self.node.name]

                else:
                    
                    self.widget = ParamChoiceWidget(self.node.name, options, default_value, optional=is_optional_param)
                    
                    self.widget_dicts[self.topic][self.node.name] = self.widget.widget
                   
            elif node.secondary_type=='string':

                if self.node.name in self.widget_dicts[self.topic]:

                    widget = self.widget_dicts[self.topic][self.node.name]

                    self.widget = ParamTextWidget(self.node.name, optional=is_optional_param, widget=widget)

                else:
                
                    self.widget = ParamTextWidget(self.node.name, optional=is_optional_param)
                    
                    self.widget_dicts[self.topic][self.node.name] = self.widget.widget
                   
            elif node.secondary_type=='string_sample':

                if self.node.name in self.widget_dicts[self.topic]:

                    self.widget = self.widget_dicts[self.topic][self.node.name]

                else:
                
                    string_sample_node = node.children[0]
                    string_sample = string_sample_node.name
                    
                    self.widget = ParamTextWidget(self.node.name, string_sample)
                    
                    self.widget_dicts[self.topic][self.node.name] = self.widget.widget
                
        elif node.primary_type=='custom1':
            
            self.widget = Custom1()
            
            self.widget_dicts[self.topic][self.node.name] = 'custom1'

        elif node.primary_type=='param_setter':

            topic, node_name = node.name.split('/')

            widget = self.widget_dicts[topic][node_name]

            self.widget = ParamSetterWidget(self.node.name, widget, self.node.default_value)
        
        self.dynamic_widget_holder = VBox()
        
        children = [
            self.widget, 
            self.dynamic_widget_holder
        ]
        
        self.widget.children[1].observe(self._add_widgets, names=['value'])
        
        super().__init__(children=children)
        
    def _add_widgets(self, widg):
        
        # if node is choiceable param
        # and if any of the choice node has param nodes
        
        # check the choice of current widget
        # and that choice has value1
        
        # look at the choice param value1, and see if that has any children
        # then loop over those children and add them all to the new_widgets
        
        input_value = widg['new']
        
        child_node = self.node.get_child_by_name(input_value)

        new_widgets = []
        
        for child_node in self.node.children:
            
            if child_node.name == input_value and (child_node.secondary_type=='param' or child_node.secondary_type=='param_setter'):
                
                for _child_node in child_node.children:
                
                    widget = DynamicWidget(self.topic, _child_node, self.widget_dicts)
                    new_widgets.append(widget)
        
        self.dynamic_widget_holder.children = tuple(new_widgets)

    # def get_param_names(self):
        
    def set_input(self, key, value):
         
        self.recur(self, key, value)

    def recur(self, node, key, value):

        if not (isinstance(node, widgets.widget_box.VBox) or 
                isinstance(node, DynamicWidget)):
            return

        if isinstance(node, DynamicWidget):

            if key in node.widget_dicts:

                node.widget_dicts[key].value = str(value)

        for child in node.children:

            self.recur(child, key, value)


class DynamicSwitch(VBox):
    
    def __init__(self, widget1, widget2):

        if not isinstance(VBox, MetaHasTraits):

            return
        
        self.widget1 = widget1
        self.widget2 = widget2
        
        self.widget = widgets.Button()
        
        self.widget.description = "Next"
        
        self.dynamic_widget_holder = VBox()

        self.dynamic_widget_holder.children = [self.widget1]
        
        children = [
            self.dynamic_widget_holder,
            self.widget
        ]
        
        self.widget.on_click(self._switch_widgets)
        
        super().__init__(children=children)
        
    def _switch_widgets(self, widg):
        
        if self.widget.description=='Back':

            new_widget = self.widget1
            self.widget.description = "Next"
            
        else:
            
            new_widget = self.widget2
            self.widget.description = "Back"
        
        self.dynamic_widget_holder.children = [new_widget]
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import setuptools
from setuptools.command.install import install

# circleci.py version
VERSION = "v0.0.b2"

def readme():
    """print long description"""
    with open('README.rst') as f:
        return f.read()


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)

# tests_require = [
#     'IPython'
#     ]

setuptools.setup(
    name="argteller-viz",
    version=VERSION,
    description="Decorator for stylized interactive constructor using DSL parser",
    long_description=readme(),
    url="https://github.com/mozjay0619/argteller-viz",
    author="Jay Kim",
    author_email="mozjay0619@gmail.com",
    license="DSB 3-clause",
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    # install_requires=[
    #     'requests==2.18.4',
    # ],
    python_requires='>=3',
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)



